"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HypernetGovernorContract = void 0;
const objects_1 = require("@hypernetlabs/objects");
const ethers_1 = require("ethers");
const neverthrow_1 = require("neverthrow");
class HypernetGovernorContract {
    constructor(providerOrSigner, contractAddress) {
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
        this.contract = null;
        this.contract = new ethers_1.ethers.Contract(contractAddress, objects_1.GovernanceAbis.HypernetGovernor.abi, providerOrSigner);
    }
    getContractAddress() {
        var _a;
        return (0, objects_1.EthereumContractAddress)(((_a = this.contract) === null || _a === void 0 ? void 0 : _a.address) || "");
    }
    _proposalIdTracker() {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a._proposalIdTracker(), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract _proposalIdTracker()", e);
        }).map((proposalCounts) => {
            return proposalCounts.toNumber();
        });
    }
    _proposalMap(index) {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a._proposalMap(index), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract _proposalMap()", e);
        });
    }
    proposals(proposalId) {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.proposals(proposalId), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract proposals()", e);
        });
    }
    state(proposalId) {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.state(proposalId), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract state()", e);
        });
    }
    proposalDescriptions(proposalId) {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.proposalDescriptions(proposalId), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract proposalDescriptions()", e);
        });
    }
    hashProposal(registryFactoryAddress, transferCalldata, descriptionHash) {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.hashProposal([registryFactoryAddress], [0], [transferCalldata], descriptionHash), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract hashProposal()", e);
        });
    }
    propose(registryFactoryAddress, transferCalldata, name) {
        if (this.contract == null) {
            throw new Error("HypernetGovernorContract is not available");
        }
        return neverthrow_1.ResultAsync.fromPromise(this.contract["propose(address[],uint256[],bytes[],string)"]([registryFactoryAddress], [0], [transferCalldata], name), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract propose()", e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.HypernetGovernorContractError("Unable to wait for tx", e);
            });
        })
            .map(() => { });
    }
    castVote(proposalId, support) {
        var _a;
        if (this.contract == null) {
            throw new Error("HypernetGovernorContract is not available");
        }
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.castVote(proposalId, support), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract castVote()", e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.HypernetGovernorContractError("Unable to wait for tx", e);
            });
        })
            .map(() => { });
    }
    getReceipt(proposalId, voterAddress) {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.getReceipt(proposalId, voterAddress), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract getReceipt()", e);
        });
    }
    queue(proposalId) {
        if (this.contract == null) {
            throw new Error("HypernetGovernorContract is not available");
        }
        return neverthrow_1.ResultAsync.fromPromise(this.contract["queue(uint256)"](proposalId), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract queue()", e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.HypernetGovernorContractError("Unable to wait for tx", e);
            });
        })
            .map(() => { });
    }
    cancel(proposalId) {
        if (this.contract == null) {
            throw new Error("HypernetGovernorContract is not available");
        }
        return neverthrow_1.ResultAsync.fromPromise(this.contract["cancel(uint256)"](proposalId), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract cancel()", e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.HypernetGovernorContractError("Unable to wait for tx", e);
            });
        })
            .map(() => { });
    }
    execute(proposalId) {
        if (this.contract == null) {
            throw new Error("HypernetGovernorContract is not available");
        }
        return neverthrow_1.ResultAsync.fromPromise(this.contract["execute(uint256)"](proposalId), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract execute()", e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.HypernetGovernorContractError("Unable to wait for tx", e);
            });
        })
            .map(() => { });
    }
    proposalThreshold() {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.proposalThreshold(), (e) => {
            return new objects_1.HypernetGovernorContractError("Unable to call HypernetGovernorContract proposalThreshold()", e);
        });
    }
}
exports.HypernetGovernorContract = HypernetGovernorContract;
//# sourceMappingURL=HypernetGovernorContract.js.map