"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RegistryFactoryContract = void 0;
const objects_1 = require("@hypernetlabs/objects");
const ethers_1 = require("ethers");
const neverthrow_1 = require("neverthrow");
class RegistryFactoryContract {
    constructor(providerOrSigner, contractAddress) {
        this.providerOrSigner = providerOrSigner;
        this.contract = null;
        this.contract = new ethers_1.ethers.Contract(contractAddress, objects_1.GovernanceAbis.UpgradeableRegistryFactory.abi, providerOrSigner);
    }
    getContractAddress() {
        var _a;
        return (0, objects_1.EthereumContractAddress)(((_a = this.contract) === null || _a === void 0 ? void 0 : _a.address) || "");
    }
    getContract() {
        return this.contract;
    }
    addressToName(registryAddress) {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.addressToName(registryAddress), (e) => {
            return new objects_1.RegistryFactoryContractError("Unable to call factoryContract addressToName()", e);
        });
    }
    enumerableRegistries(index) {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.enumerableRegistries(index), (e) => {
            return new objects_1.RegistryFactoryContractError("Unable to call factoryContract enumerableRegistries()", e);
        });
    }
    nameToAddress(registryName) {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.nameToAddress(registryName), (e) => {
            return new objects_1.RegistryFactoryContractError("Unable to call factoryContract nameToAddress()", e);
        });
    }
    getNumberOfEnumerableRegistries() {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.getNumberOfEnumerableRegistries(), (e) => {
            return new objects_1.RegistryFactoryContractError("Unable to call factoryContract getNumberOfEnumerableRegistries()", e);
        }).map((numberOfRegistries) => numberOfRegistries.toNumber());
    }
    registrationFee() {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.registrationFee(), (e) => {
            return new objects_1.RegistryFactoryContractError("Unable to call factoryContract registrationFee()", e);
        });
    }
    createRegistryByToken(name, symbol, registrarAddress, enumerable) {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.createRegistryByToken(name, symbol, registrarAddress, enumerable), (e) => {
            return new objects_1.RegistryFactoryContractError("Unable to call factoryContract createRegistryByToken()", e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.RegistryFactoryContractError("Unable to wait for tx", e);
            });
        })
            .map(() => { });
    }
    modules(index) {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.modules(index), (e) => {
            return new objects_1.RegistryFactoryContractError("Unable to call factoryContract modules()", e);
        });
    }
    getModuleName(moduleAddress) {
        const moduleABI = [
            {
                inputs: [],
                name: "name",
                outputs: [
                    {
                        internalType: "string",
                        name: "",
                        type: "string",
                    },
                ],
                stateMutability: "view",
                type: "function",
            },
        ];
        const moduleContract = new ethers_1.ethers.Contract(moduleAddress, moduleABI, this.providerOrSigner);
        return neverthrow_1.ResultAsync.fromPromise(moduleContract === null || moduleContract === void 0 ? void 0 : moduleContract.name(), (e) => {
            return new objects_1.RegistryFactoryContractError("Unable to call moduleContract name()", e);
        });
    }
    getNumberOfModules() {
        var _a;
        return neverthrow_1.ResultAsync.fromPromise((_a = this.contract) === null || _a === void 0 ? void 0 : _a.getNumberOfModules(), (e) => {
            return new objects_1.RegistryFactoryContractError("Unable to call factoryContract getNumberOfModules()", e);
        }).map((numberOfRegistries) => numberOfRegistries.toNumber());
    }
}
exports.RegistryFactoryContract = RegistryFactoryContract;
//# sourceMappingURL=RegistryFactoryContract.js.map